@@ -228,7 +228,7 @@ ex & ex::operator[](size_t i)
 /** Left hand side of relational expression. */
 ex ex::lhs() const
 {
-	if (!is_a<relational>(*this))
+	if (!is_exactly_a<relational>(*this))
 		throw std::runtime_error("ex::lhs(): not a relation");
 	return bp->op(0);
 }
@@ -236,7 +236,7 @@ ex ex::lhs() const
 /** Right hand side of relational expression. */
 ex ex::rhs() const
 {
-	if (!is_a<relational>(*this))
+	if (!is_exactly_a<relational>(*this))
 		throw std::runtime_error("ex::rhs(): not a relation");
 	return bp->op(1);
 }
@@ -244,7 +244,7 @@ ex ex::rhs() const
 /** Check whether expression is a polynomial. */
 bool ex::is_polynomial(const ex & vars) const
 {
-	if (is_a<lst>(vars)) {
+	if (is_exactly_a<lst>(vars)) {
 		const lst & varlst = ex_to<lst>(vars);
 		for (lst::const_iterator i=varlst.begin(); i!=varlst.end(); ++i)
 			if (!bp->is_polynomial(*i))
@@ -261,8 +261,7 @@ bool ex::is_zero() const {
 #endif
         if (!is_exactly_a<numeric>(*this))
                 return is_equal(_ex0);
-        numeric num = ex_to<numeric>(*this);
-        return num.is_zero();
+        return ex_to<numeric>(*this).is_zero();
 }
 
 /** Check whether expression is zero or zero matrix. */
@@ -272,7 +271,7 @@ bool ex::is_zero_matrix() const
 		return  true;
 	else {
 		ex e = evalm();
-		return is_a<matrix>(e) && ex_to<matrix>(e).is_zero_matrix();
+		return is_exactly_a<matrix>(e) && ex_to<matrix>(e).is_zero_matrix();
 	}
 }
 
@@ -280,8 +279,8 @@ bool ex::is_integer_one() const
 {
     if (!is_exactly_a<numeric>(*this))
         return false;
-    numeric num = ex_to<numeric>(*this);
-    return num.is_integer() && num.is_equal(*_num1_p);
+    return ex_to<numeric>(*this).is_integer()
+        and ex_to<numeric>(*this).is_equal(*_num1_p);
 }
   
 bool ex::is_integer_pmone() const
@@ -289,14 +288,15 @@ bool ex::is_integer_pmone() const
     if (!is_exactly_a<numeric>(*this))
         return false;
     numeric num = ex_to<numeric>(*this);
-    return ((num.is_integer()) &&
-            (num.is_equal(*_num1_p) || num.is_equal(*_num_1_p)));
+    return ((ex_to<numeric>(*this).is_integer())
+        and (ex_to<numeric>(*this).is_equal(*_num1_p)
+            or ex_to<numeric>(*this).is_equal(*_num_1_p)));
 }
 
 size_t ex::nsymbols() const
 {
 	int res = 0;
-	if (is_a<symbol>(*this)) {
+	if (is_exactly_a<symbol>(*this)) {
 		res=1;
 	} else {
 		for (size_t i=0; i < nops(); i++)
