@@ -761,7 +761,7 @@ ex mul::eval(int level) const
 		numeric oc = *_num1_p;
 		bool something_changed = false;
 		while (i!=last) {
-			if (likely(! (is_a<add>(i->rest) && i->coeff.is_integer_one()))) {
+			if (likely(! (is_exactly_a<add>(i->rest) && i->coeff.is_integer_one()))) {
 				// power::eval has such a rule, no need to handle powers here
 				++i;
 				continue;
@@ -858,7 +858,7 @@ ex mul::eval_exponentials() const
 	}
 
 	ex new_exp = exp(exp_arg);
-	if (is_a<numeric>(new_exp))
+	if (is_exactly_a<numeric>(new_exp))
 		oc = oc.mul(ex_to<numeric>(new_exp));
 	else
 		s->push_back(expair(new_exp, _ex1));
@@ -943,7 +943,7 @@ ex mul::evalm() const
 {
 	// numeric*matrix
 	if (seq.size() == 1 && seq[0].coeff.is_equal(_ex1)
-	 && is_a<matrix>(seq[0].rest))
+	 && is_exactly_a<matrix>(seq[0].rest))
 		return ex_to<matrix>(seq[0].rest).mul(ex_to<numeric>(overall_coeff));
 
 	// Evaluate children first, look whether there are any matrices at all
@@ -959,7 +959,7 @@ ex mul::evalm() const
 	while (i != end) {
 		const ex &m = recombine_pair_to_ex(*i).evalm();
 		s->push_back(split_ex_to_pair(m));
-		if (is_a<matrix>(m)) {
+		if (is_exactly_a<matrix>(m)) {
 			have_matrix = true;
 			the_matrix = s->end() - 1;
 		}
@@ -1081,7 +1081,7 @@ bool mul::has(const ex & pattern, unsigned options) const
 {
 	if(!(options&has_options::algebraic))
 		return basic::has(pattern,options);
-	if(is_a<mul>(pattern)) {
+	if(is_exactly_a<mul>(pattern)) {
 		lst repls;
 		int nummatches = std::numeric_limits<int>::max();
 		std::vector<bool> subsed(nops(), false);
@@ -1405,7 +1405,7 @@ ex mul::expand(unsigned options) const
 	{
 	// trivial case: expanding the monomial (~ 30% of all calls)
 		epvector::const_iterator i = seq.begin(), seq_end = seq.end();
-		while ((i != seq.end()) &&  is_a<symbol>(i->rest) && i->coeff.info(info_flags::integer))
+		while ((i != seq.end()) &&  is_exactly_a<symbol>(i->rest) && i->coeff.info(info_flags::integer))
 			++i;
 		if (i == seq_end) {
 			setflag(status_flags::expanded);
