@@ -201,7 +201,7 @@ void power::do_print_latex(const print_latex & c, unsigned level) const
 
 		if (!(-exponent).is_integer_one()) {
     		        c.s << "^{";
-			bool exp_parenthesis = is_a<power>(exponent);
+			bool exp_parenthesis = is_exactly_a<power>(exponent);
 			if (exp_parenthesis)
 				c.s << "\\left(";
 			exponent.print(c, 0);
@@ -258,7 +258,7 @@ void power::do_print_csrc(const print_csrc & c, unsigned level) const
 
 	// Integer powers of symbols are printed in a special, optimized way
 	if (exponent.info(info_flags::integer)
-	 && (is_a<symbol>(basis) || is_a<constant>(basis))) {
+	 && (is_exactly_a<symbol>(basis) || is_exactly_a<constant>(basis))) {
 		int exp = ex_to<numeric>(exponent).to_int();
 		if (exp > 0)
 			c.s << '(';
@@ -478,7 +478,7 @@ ex power::eval(int level) const
 	}
 
 	// ^(\infty, x)
-	if (is_a<infinity>(ebasis)) {
+	if (is_exactly_a<infinity>(ebasis)) {
 		const infinity & basis_inf = ex_to<infinity>(ebasis);
 		if (eexponent.nsymbols()>0)
 			throw(std::domain_error("power::eval(): pow(Infinity, f(x)) is not defined."));
@@ -496,7 +496,7 @@ ex power::eval(int level) const
 	}
 
 	// ^(x, \infty)
-	if (is_a<infinity>(eexponent)) {
+	if (is_exactly_a<infinity>(eexponent)) {
 		const infinity & exp_inf = ex_to<infinity>(eexponent);
 		if (exp_inf.is_unsigned_infinity())
 			throw(std::domain_error("power::eval(): pow(x, unsigned_infinity) is not defined."));
@@ -711,7 +711,7 @@ ex power::eval(int level) const
 		// ^(nc,c1) -> ncmul(nc,nc,...) (c1 positive integer, unless nc is a matrix)
 		if (ebasis.return_type() != return_types::commutative &&
                     num_exponent->is_pos_integer() &&
-                    !is_a<matrix>(ebasis)) {
+                    !is_exactly_a<matrix>(ebasis)) {
 			return ncmul(exvector(num_exponent->to_int(), ebasis), true);
 		}
 	}
@@ -749,7 +749,7 @@ ex power::evalm() const
 {
 	const ex ebasis = basis.evalm();
 	const ex eexponent = exponent.evalm();
-	if (is_a<matrix>(ebasis)) {
+	if (is_exactly_a<matrix>(ebasis)) {
 		if (is_exactly_a<numeric>(eexponent)) {
 			return (new matrix(ex_to<matrix>(ebasis).pow(eexponent)))->setflag(status_flags::dynallocated);
 		}
@@ -761,7 +761,7 @@ bool power::has(const ex & other, unsigned options) const
 {
 	if (!(options & has_options::algebraic))
 		return basic::has(other, options);
-	if (!is_a<power>(other))
+	if (!is_exactly_a<power>(other))
 		return basic::has(other, options);
 	if (!exponent.info(info_flags::integer)
 			|| !other.op(1).info(info_flags::integer))
@@ -890,7 +890,7 @@ ex power::imag_part() const
  *  @see ex::diff */
 ex power::derivative(const symbol & s) const
 {
-	if (is_a<numeric>(exponent)) {
+	if (is_exactly_a<numeric>(exponent)) {
 		// D(b^r) = r * b^(r-1) * D(b) (faster than the formula below)
 		epvector newseq;
 		newseq.reserve(2);
@@ -938,7 +938,7 @@ tinfo_t power::return_type_tinfo() const
 
 ex power::expand(unsigned options) const
 {
-	if (is_a<symbol>(basis) && exponent.info(info_flags::integer)) {
+	if (is_exactly_a<symbol>(basis) && exponent.info(info_flags::integer)) {
 		// A special case worth optimizing.
 		setflag(status_flags::expanded);
 		return *this;
